@@ -1,13 +1,11 @@
 use crate::basic_sharing::{from_secrets, reconstruct_secrets};
-// use crypto::digest::Digest;
-// use crypto::sha3::Sha3;
+use sha3::digest::generic_array::GenericArray;
+use sha3::digest::FixedOutput;
+use sha3::Digest;
 use std::convert::TryFrom;
 use std::fs::File;
 use std::io::{Cursor, Read, Write};
 use std::path::{Path, PathBuf};
-use sha3::Digest;
-use sha3::digest::FixedOutput;
-use sha3::digest::generic_array::GenericArray;
 
 const NUM_FIRST_BYTES_FOR_VERIFY: usize = 32;
 pub const READ_SEGMENT_SIZE: usize = 8_192; // 8 KB, which has shown optimal perforamnce
@@ -57,6 +55,10 @@ impl Secret {
         }
     }
 
+    pub fn is_empty(&self) -> Result<bool, Error> {
+        Ok(self.len()? == 0)
+    }
+
     /// Calculates and returns the Sha3-512 hash of the first 64 bytes of the secret.
     ///
     /// This is mainly used for verifying secret reconstruction, where the chances of incorrect
@@ -64,7 +66,7 @@ impl Secret {
     ///
     /// If $secret.len() is less than 64 bytes, then only $secret.len() number of bytes is used.
     pub fn calculate_hash(&self) -> Result<Vec<u8>, Error> {
-        let mut hasher_output = GenericArray::from_slice(&[0u8; 64]).clone();
+        let mut hasher_output = *GenericArray::from_slice(&[0u8; 64]);
         let mut hasher = sha3::Sha3_512::new();
         let len = self.len()?;
         let hash_input_num_bytes = if len < (NUM_FIRST_BYTES_FOR_VERIFY as u64) {
@@ -77,8 +79,7 @@ impl Secret {
 
         match self {
             Secret::InFile(ref path) => {
-                let f = File::open(path)
-                    .map_err(|e| Error::secret_file_error(self, e))?;
+                let f = File::open(path).map_err(|e| Error::secret_file_error(self, e))?;
                 f.take(hash_input_num_bytes as u64)
                     .read_to_end(&mut input_vec)
                     .map_err(|e| Error::secret_file_error(self, e))?;
@@ -106,167 +107,6 @@ impl Secret {
         Ok(self.calculate_hash()? == hash.to_vec())
     }
 
-    /// Reconstructs a secret from a given list of shares.
-    ///
-    /// **verify**: If true, a hash is assumed to exist at the end of the secret and will be used
-    ///             to verify secret reconstruction. NOTE: This will fail if the secret was not
-    ///             shared with verify set to true.
-    pub fn reconstruct(&mut self, srcs: Vec<Vec<u8>>, verify: bool) -> Result<(), Error> {
-        let src_len = srcs[0].len() as u64;
-        let mut srcs = srcs
-            .into_iter()
-            .map(|share| Box::new(Cursor::new(share)) as Box<dyn Read>)
-            .collect();
-        self.reconstruct_from_srcs(&mut srcs, src_len, verify)
-    }
-
-    /// Reconstructs a secret from a given list of srcs. The srcs should all read the same number
-    /// of bytes.
-    /// **src_len** MUST be an accurate length of the shares
-    pub fn reconstruct_from_srcs(
-        &mut self,
-        mut srcs: &mut Vec<Box<dyn Read>>,
-        src_len: u64,
-        verify: bool,
-    ) -> Result<(), Error> {
-
-        // This is to avoid multiple reference issues.
-        let path = match self {
-            Secret::InFile(path) => String::from(path.to_str().unwrap()),
-            _ => String::from("<Cannot be determined>"),
-        };
-
-        let to_points = |vec: Vec<u8>, segment_num: u8| -> Vec<(u8, u8)> {
-            vec.into_iter().map(|val| (segment_num, val)).collect()
-        };
-        let get_shares = |num_bytes: usize,
-                          srcs: &mut Vec<Box<dyn Read>>,
-                          x_vals: &Vec<u8>|
-         -> Result<Vec<Vec<(u8, u8)>>, Error> {
-            let mut segments: Vec<Vec<(u8, u8)>> = Vec::with_capacity(srcs.len());
-
-            // Read in one segment size from each share
-            for (src, x_val) in srcs.into_iter().zip(x_vals) {
-                let mut buf: Vec<u8> = Vec::with_capacity(num_bytes as usize);
-                src.take(num_bytes as u64)
-                    .read_to_end(&mut buf)
-                    .map_err(|e| Error::FileError(path.clone(), e))?;
-                segments.push(to_points(buf, *x_val));
-            }
-            Ok(segments)
-        };
-
-        // First, get the first byte from each share, which is the x value for those shares
-        let mut buf = Vec::with_capacity(1);
-        let mut x_vals = Vec::with_capacity(srcs.len());
-        for src in srcs.into_iter() {
-            buf.clear();
-            src.take(1)
-                .read_to_end(&mut buf)
-                .map_err(|e| Error::secret_file_error(self, e))?;
-            x_vals.push(buf[0]);
-        }
-
-        let src_len = if verify {
-            u64::try_from((src_len as i64) - 64 - 1).unwrap()
-        } else {
-            u64::try_from((src_len as i64) - 1).unwrap()
-        };
-
-        let segments_to_read = if src_len % (READ_SEGMENT_SIZE as u64) != 0 {
-            (src_len / (READ_SEGMENT_SIZE as u64)) + 1
-        } else {
-            src_len / (READ_SEGMENT_SIZE as u64)
-        } as usize;
-
-        // 64 is the hash len, which we don't want to include in the output secret, just to verify
-        // that the secret was reconstructed properly. This should never underflow if valid shares
-        // are given since src_len will always be N + 64 where N is the share size
-
-        let mut dest: Box<dyn Write> = match self {
-            Secret::InMemory(ref mut vec) => Box::new(vec) as Box<dyn Write>,
-            Secret::InFile(ref path) => Box::new(
-                File::create(path)
-                    .map_err(|e| Error::secret_file_error(self, e))?
-            ) as Box<dyn Write>,
-        };
-
-        // Now read in segments and compute the secrets and write the secrets to the destination
-        // Skip the last segment for now since it includes the appended hash
-        if segments_to_read > 0 {
-            for _ in 0..(segments_to_read - 1) {
-                let segments = get_shares(READ_SEGMENT_SIZE, srcs, &x_vals)?;
-                // Now segments has a segment from each share src, reconstruct the secret up to that
-                // point and write it to the destination
-                dest.write_all(reconstruct_secrets(segments)?.as_slice())
-                    .map_err(|e| Error::FileError(path.clone(), e))?;
-            }
-        }
-
-        // If the secret isn't exactly divisible by READ_SEGMENT_SIZE, read the leftover bytes
-        // and write the computed secret into dest
-        let remaining_bytes = (src_len % (READ_SEGMENT_SIZE as u64)) as usize;
-        if remaining_bytes > 0 {
-            let last_segments = get_shares(remaining_bytes, &mut srcs, &x_vals)?;
-            dest.write_all(reconstruct_secrets(last_segments)?.as_slice())
-                .map_err(|e| Error::FileError(path.clone(), e))?;
-        }
-
-        if verify {
-            // Now read in the hash
-            let hash_segments = get_shares(512, &mut srcs, &x_vals)?;
-            let recon_hash = reconstruct_secrets(hash_segments)?;
-            // Drop dest since if it is a file, we will be re-opening it to read from it to
-            // calculate the hash. Ensure output is flushed
-            dest.flush().ok();
-            std::mem::drop(dest);
-            if !self.verify(recon_hash.as_slice())? {
-                let calc_hash_hex = hex::encode(self.calculate_hash()?);
-                let orig_hash_hex = hex::encode(&recon_hash);
-                return Err(Error::VerificationFailure(orig_hash_hex, calc_hash_hex));
-            }
-        }
-        Ok(())
-    }
-
-    /// Performs the reconstruction of the shares from files with in the given **dir** with the give **stem**
-    pub fn reconstruct_from_files<T: AsRef<Path>>(
-        &mut self,
-        dir: T,
-        stem: &str,
-        shares_required: u8,
-        verify: bool,
-    ) -> Result<(), Error> {
-        let share_paths = generate_share_file_paths(&dir, stem, shares_required);
-        let share_files: Vec<Result<File, Error>> = share_paths
-            .into_iter()
-            .map(|path| File::open(&path).map_err(|e| Error::FileError(String::from(&path), e)))
-            .collect();
-
-        // Check that all the share files opened properly
-        let mut unwrapped_share_files: Vec<File> = Vec::with_capacity(share_files.len());
-        for file in share_files {
-            unwrapped_share_files.push(file?);
-        }
-        // An error here would be extremely rare, so just panic.
-        let len = unwrapped_share_files[0]
-            .metadata()
-            .expect("An error occured after opening file for read.")
-            .len();
-
-        // Now map the files to a dyn Read, which needed to wait till we got the len since Read
-        // doesn't have a len method.
-        let mut share_files: Vec<Box<dyn Read>> = unwrapped_share_files
-            .into_iter()
-            .map(|file| Box::new(file) as Box<dyn Read>)
-            .collect();
-
-        let mut secret_path = dir.as_ref().to_path_buf();
-        secret_path.push(stem);
-
-        self.reconstruct_from_srcs(&mut share_files, len, verify)
-    }
-
     /// Unwrap and return the inner vec, consuming the secret.
     ///
     /// This will panic if the underlying secret is InFile.
@@ -290,7 +130,7 @@ impl Secret {
     /// Returns None if the inner value is a path, else returns the secret.
     pub fn try_unwrap_vec(&mut self) -> Option<Vec<u8>> {
         match self {
-            Secret::InMemory(ref mut secret) => Some(std::mem::replace(secret, Vec::new())),
+            Secret::InMemory(ref mut secret) => Some(std::mem::take(secret)),
             _ => None,
         }
     }
@@ -314,9 +154,8 @@ impl Secret {
     pub fn unwrap_to_vec(self) -> Result<Vec<u8>, Error> {
         match self {
             Secret::InMemory(secret) => Ok(secret),
-            Secret::InFile(path) => {
-                std::fs::read(&path).map_err(|e| Error::FileError(String::from(path.to_str().unwrap()), e))
-            }
+            Secret::InFile(path) => std::fs::read(&path)
+                .map_err(|e| Error::FileError(String::from(path.to_str().unwrap()), e)),
         }
     }
 
@@ -327,7 +166,9 @@ impl Secret {
     pub fn unwrap_to_vec_clone(&self) -> Result<Vec<u8>, Error> {
         match self {
             Secret::InMemory(ref secret) => Ok(secret.clone()),
-            Secret::InFile(path) => std::fs::read(path).map_err(|e| Error::secret_file_error(self, e))
+            Secret::InFile(path) => {
+                std::fs::read(path).map_err(|e| Error::secret_file_error(self, e))
+            }
         }
     }
 }
@@ -380,7 +221,7 @@ impl std::iter::Iterator for SecretIterator {
                 _ => e.into(),
             }));
         }
-        if result.len() == 0 {
+        if result.is_empty() {
             return None;
         }
         Some(Ok(result))
@@ -395,9 +236,9 @@ impl std::iter::Iterator for SecretIterator {
 ///
 /// **verify**: If true, a hash is calculated from the secret and placed at the end to be used
 ///             to verify reconstruction of the secret.
-pub fn share_to_writables(
+pub fn share_to_writables<'a>(
     secret: Secret,
-    mut dests: &mut Vec<Box<dyn Write>>,
+    dests: &mut Vec<Box<dyn Write + 'a>>,
     shares_required: u8,
     shares_to_create: u8,
     verify: bool,
@@ -405,8 +246,8 @@ pub fn share_to_writables(
     // This just writes each corresponding share_list in share_lists to a dest in dests. This
     // is written here as a closure since it's used at two different points in this function
     let share_lists_to_dests =
-        |lists: Vec<Vec<(u8, u8)>>, mut dests: &mut Vec<Box<dyn Write>>| -> Result<(), Error> {
-            for (share_list, dest) in lists.into_iter().zip((&mut dests).into_iter()) {
+        |lists: Vec<Vec<(u8, u8)>>, dests: &mut Vec<Box<dyn Write + 'a>>| -> Result<(), Error> {
+            for (share_list, dest) in lists.into_iter().zip(dests.iter_mut()) {
                 dest.write_all(
                     share_list
                         .into_iter()
@@ -419,8 +260,8 @@ pub fn share_to_writables(
         };
 
     // Write out the x value to each dest that will be used for each following point
-    for (x_val, dest) in dests.into_iter().enumerate() {
-        dest.write(&[(x_val + 1) as u8])?;
+    for (x_val, dest) in dests.iter_mut().enumerate() {
+        dest.write_all(&[(x_val + 1) as u8])?;
     }
 
     if dests.len() < (shares_to_create as usize) {
@@ -435,14 +276,14 @@ pub fn share_to_writables(
         // if the secret is a file and a reading error occured during iteration
         let secret_segment = secret_segment?;
 
-        if secret_segment.len() > 0 {
+        if !secret_segment.is_empty() {
             let share_lists = from_secrets(
                 secret_segment.as_slice(),
                 shares_required,
                 shares_to_create,
                 None,
             )?;
-            share_lists_to_dests(share_lists, &mut dests)?;
+            share_lists_to_dests(share_lists, dests)?;
         }
 
         std::mem::drop(secret_segment); // ensure the memory DOES get dropped
@@ -455,11 +296,11 @@ pub fn share_to_writables(
         let share_lists = from_secrets(&hash, shares_required, shares_to_create, None)?;
 
         // The shares for the hash have been created, write them all to dests
-        share_lists_to_dests(share_lists, &mut dests)?;
+        share_lists_to_dests(share_lists, dests)?;
     }
 
     // Flush writes to all dests to ensure all bytes are written
-    for dest in (&mut dests).into_iter() {
+    for dest in dests.iter_mut() {
         dest.flush().ok();
     }
     Ok(())
@@ -539,6 +380,156 @@ pub fn share_to_files<T: AsRef<Path>>(
     )
 }
 
+/// Reconstructs a secret from a given list of shares.
+///
+/// **verify**: If true, a hash is assumed to exist at the end of the secret and will be used
+///             to verify secret reconstruction. NOTE: This will fail if the secret was not
+///             shared with verify set to true.
+pub fn reconstruct(secret: &mut Secret, srcs: Vec<Vec<u8>>, verify: bool) -> Result<(), Error> {
+    let src_len = srcs[0].len() as u64;
+    let mut srcs = srcs
+        .into_iter()
+        .map(|share| Box::new(Cursor::new(share)) as Box<dyn Read>)
+        .collect();
+    reconstruct_from_srcs(secret, &mut srcs, src_len, verify)
+}
+
+/// Reconstructs a secret from a given list of srcs. The srcs should all read the same number
+/// of bytes.
+/// **src_len** MUST be an accurate length of the shares
+pub fn reconstruct_from_srcs<'a>(
+    secret: &mut Secret,
+    srcs: &mut Vec<Box<dyn Read + 'a>>,
+    src_len: u64,
+    verify: bool,
+) -> Result<(), Error> {
+    // This is to avoid multiple reference issues.
+    let to_points = |vec: Vec<u8>, segment_num: u8| -> Vec<(u8, u8)> {
+        vec.into_iter().map(|val| (segment_num, val)).collect()
+    };
+    let get_shares = |num_bytes: usize,
+                      srcs: &mut Vec<Box<dyn Read + 'a>>,
+                      x_vals: &Vec<u8>|
+     -> Result<Vec<Vec<(u8, u8)>>, Error> {
+        let mut segments: Vec<Vec<(u8, u8)>> = Vec::with_capacity(srcs.len());
+
+        // Read in one segment size from each share
+        for (src, x_val) in srcs.iter_mut().zip(x_vals) {
+            let mut buf: Vec<u8> = Vec::with_capacity(num_bytes as usize);
+            src.take(num_bytes as u64).read_to_end(&mut buf)?;
+            segments.push(to_points(buf, *x_val));
+        }
+        Ok(segments)
+    };
+
+    // First, get the first byte from each share, which is the x value for those shares
+    let mut buf = Vec::with_capacity(1);
+    let mut x_vals = Vec::with_capacity(srcs.len());
+    for src in srcs.iter_mut() {
+        buf.clear();
+        src.take(1)
+            .read_to_end(&mut buf)
+            .map_err(|e| Error::secret_file_error(secret, e))?;
+        x_vals.push(buf[0]);
+    }
+
+    let src_len = if verify {
+        u64::try_from((src_len as i64) - 64 - 1).unwrap()
+    } else {
+        u64::try_from((src_len as i64) - 1).unwrap()
+    };
+
+    let segments_to_read = if src_len % (READ_SEGMENT_SIZE as u64) != 0 {
+        (src_len / (READ_SEGMENT_SIZE as u64)) + 1
+    } else {
+        src_len / (READ_SEGMENT_SIZE as u64)
+    } as usize;
+
+    // 64 is the hash len, which we don't want to include in the output secret, just to verify
+    // that the secret was reconstructed properly. This should never underflow if valid shares
+    // are given since src_len will always be N + 64 where N is the share size
+
+    let mut dest: Box<dyn Write> = match secret {
+        Secret::InMemory(ref mut vec) => Box::new(vec) as Box<dyn Write>,
+        Secret::InFile(ref path) => {
+            Box::new(File::create(path).map_err(|e| Error::secret_file_error(secret, e))?)
+                as Box<dyn Write>
+        }
+    };
+
+    // Now read in segments and compute the secrets and write the secrets to the destination
+    // Skip the last segment for now since it includes the appended hash
+    let mut curr_len = src_len;
+    if segments_to_read > 0 {
+        while curr_len > 0 {
+            let segment_size = if curr_len < (READ_SEGMENT_SIZE as u64) {
+                curr_len as usize
+            } else {
+                READ_SEGMENT_SIZE
+            };
+            let segments = get_shares(segment_size, srcs, &x_vals)?;
+            // Now segments has a segment from each share src, reconstruct the secret up to that
+            // point and write it to the destination
+            dest.write_all(reconstruct_secrets(segments)?.as_slice())?;
+            curr_len = curr_len.saturating_sub(READ_SEGMENT_SIZE as u64);
+        }
+    }
+
+    if verify {
+        // Now read in the hash
+        let hash_segments = get_shares(512, srcs, &x_vals)?;
+        let recon_hash = reconstruct_secrets(hash_segments)?;
+        // Drop dest since if it is a file, we will be re-opening it to read from it to
+        // calculate the hash. Ensure output is flushed
+        dest.flush().ok();
+        std::mem::drop(dest);
+        if !secret.verify(recon_hash.as_slice())? {
+            let calc_hash_hex = hex::encode(secret.calculate_hash()?);
+            let orig_hash_hex = hex::encode(&recon_hash);
+            return Err(Error::VerificationFailure(orig_hash_hex, calc_hash_hex));
+        }
+    }
+    Ok(())
+}
+
+/// Performs the reconstruction of the shares from files with in the given **dir** with the give **stem**
+pub fn reconstruct_from_files<T: AsRef<Path>>(
+    secret: &mut Secret,
+    dir: T,
+    stem: &str,
+    shares_required: u8,
+    verify: bool,
+) -> Result<(), Error> {
+    let share_paths = generate_share_file_paths(&dir, stem, shares_required);
+    let share_files: Vec<Result<File, Error>> = share_paths
+        .into_iter()
+        .map(|path| File::open(&path).map_err(|e| Error::FileError(String::from(&path), e)))
+        .collect();
+
+    // Check that all the share files opened properly
+    let mut unwrapped_share_files: Vec<File> = Vec::with_capacity(share_files.len());
+    for file in share_files {
+        unwrapped_share_files.push(file?);
+    }
+    // An error here would be extremely rare, so just panic.
+    let len = unwrapped_share_files[0]
+        .metadata()
+        .expect("An error occured after opening file for read.")
+        .len();
+
+    // Now map the files to a dyn Read, which needed to wait till we got the len since Read
+    // doesn't have a len method.
+    let mut share_files: Vec<Box<dyn Read>> = unwrapped_share_files
+        .into_iter()
+        .map(|file| Box::new(file) as Box<dyn Read>)
+        .collect();
+
+    let mut secret_path = dir.as_ref().to_path_buf();
+    secret_path.push(stem);
+
+    reconstruct_from_srcs(secret, &mut share_files, len, verify)
+}
+
 #[derive(Debug)]
 pub enum Error {
     ReconstructionNotEqual,
@@ -599,7 +590,7 @@ Calculated Hash: {}",
             ),
             Error::FileError(path, source) => {
                 write!(f, "File with path '{}' could not be used: {}", path, source)
-            },
+            }
             Error::IOError(source) => {
                 write!(f, "IOError: '{}'", source)
             }
@@ -612,12 +603,11 @@ Calculated Hash: {}",
 
 impl std::error::Error for Error {}
 
-
 impl Error {
     fn secret_file_error(secret: &Secret, e: std::io::Error) -> Self {
         match secret {
             Secret::InFile(ref path) => Error::FileError(String::from(path.to_str().unwrap()), e),
-            _ => Error::FileError(String::from("<Non-File-based IO Error>"), e)
+            _ => Error::FileError(String::from("<Non-File-based IO Error>"), e),
         }
     }
 }
@@ -669,8 +659,7 @@ mod tests {
 
         let mut recon = Secret::empty_in_memory();
 
-        recon
-            .reconstruct_from_files(dir, stem, num_shares, true)
+        reconstruct_from_files(&mut recon, dir, stem, num_shares, true)
             .map_err(|_| ())
             .ok();
 
@@ -699,7 +688,7 @@ mod tests {
         )
         .unwrap();
         let mut recon = Secret::empty_in_memory();
-        recon.reconstruct(shares, true).unwrap();
+        reconstruct(&mut recon, shares, true).unwrap();
 
         assert_eq!(secret, recon.unwrap_vec());
     }
@@ -718,9 +707,7 @@ mod tests {
         .unwrap();
         shares.as_mut_slice().shuffle(&mut thread_rng());
         let mut recon_secret = Secret::empty_in_memory_with_capacity(secret.len());
-        recon_secret
-            .reconstruct(shares[0..3].to_vec(), true)
-            .unwrap();
+        reconstruct(&mut recon_secret, shares[0..3].to_vec(), true).unwrap();
 
         assert_eq!(secret, recon_secret.unwrap_vec());
     }
